import axiomic.frontend.config as _config
import axiomic.frontend.text as _text
import axiomic.frontend.image as _image
import axiomic.errors as _errors
import axiomic.frontend.params as _params
import axiomic.frontend.functions as _functions

import axiomic.models as models



# axiomic operations
Text = _text.Text
TextType = _text.TextType

# Images
Image = _image.Image


# Errors
ThoughtCrime = _errors.ThoughtCrime
GraphError = _errors.GraphError


# Functions
infer = _functions.infer
format = _functions.format


# Params
P = _params.P