# import axiomic.models.context as _context
# 
# 
# register_global_provider = _context.register_global_provider
# register_global_ref_provider = _context.register_global_ref_provider
# infer_context = _context.infer_context
# 
# get_provider_provider = _context.get_provider_provider
# print_inferred_context = _context.print_inferred_context
# 